package gui;

import java.awt.event.ItemEvent;
import java.awt.event.KeyEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Event;
import java.awt.BorderLayout;

import javax.sound.midi.SysexMessage;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.KeyStroke;
import java.awt.Point;

import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JTextArea;
import java.awt.Dimension;
import javax.swing.JTree;
import javax.swing.BoxLayout;
import java.awt.CardLayout;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;
import javax.swing.JComboBox;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.awt.Insets;

public class GUI {

	private JFrame jFrame = null;  //  @jve:decl-index=0:visual-constraint="10,10"
	private JPanel jContentPane = null;
	private JMenuBar jJMenuBar = null;
	private JMenu fileMenu = null;
	private JMenu editMenu = null;
	private JMenu helpMenu = null;
	private JMenuItem exitMenuItem = null;
	private JMenuItem aboutMenuItem = null;
	private JMenuItem cutMenuItem = null;
	private JMenuItem copyMenuItem = null;
	private JMenuItem pasteMenuItem = null;
	private JMenuItem saveMenuItem = null;
	private JDialog aboutDialog = null;  //  @jve:decl-index=0:visual-constraint="446,16"
	private JPanel aboutContentPane = null;
	private JLabel aboutVersionLabel = null;
	private JMenu ViewMenu = null;
	private JMenuItem NewMenuItem = null;
	private JMenu openMenu = null;
	private JMenuItem openBandMenuItem = null;
	private JMenuItem openMusicianMenuItem = null;
    private JPanel northPanel = null;
    private JPanel westPanel = null;
    private JScrollPane imageScrollPane = null;
    private JPanel eastPanel = null;
    private JPanel southPanel = null;
    private JButton startButton = null;
    private JButton previousButton = null;
    private JButton nextButton = null;
    private JLabel artistLabel = null;
    private JComboBox artistComboBox = null;
    private JLabel imageLabel = null;
    private JTextArea infoTextArea = null;
    private JScrollPane eastScrollPane = null;
    
    private int selectedIndex = -1;
    /**
	 * This method initializes jFrame
	 * 
	 * @return javax.swing.JFrame
	 */
	private JFrame getJFrame() {
		if (jFrame == null) {
			jFrame = new JFrame();
			jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			jFrame.setJMenuBar(getJJMenuBar());
			jFrame.setSize(392, 276);
			jFrame.setContentPane(getJContentPane());
			jFrame.setTitle("Rock 'n' Roll");
		}
		return jFrame;
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getNorthPanel(), BorderLayout.NORTH);
			jContentPane.add(getWestPanel(), BorderLayout.WEST);
			jContentPane.add(getImageScrollPane(), BorderLayout.CENTER);
			jContentPane.add(getEastPanel(), BorderLayout.EAST);
			jContentPane.add(getSouthPanel(), BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jJMenuBar	
	 * 	
	 * @return javax.swing.JMenuBar	
	 */
	private JMenuBar getJJMenuBar() {
		if (jJMenuBar == null) {
			jJMenuBar = new JMenuBar();
			jJMenuBar.add(getFileMenu());
			jJMenuBar.add(getEditMenu());
			jJMenuBar.add(getViewMenu());
			jJMenuBar.add(getHelpMenu());
		}
		return jJMenuBar;
	}

	/**
	 * This method initializes jMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getFileMenu() {
		if (fileMenu == null) {
			fileMenu = new JMenu();
			fileMenu.setText("File");
			fileMenu.add(getNewMenuItem());
			fileMenu.add(getOpenMenu());
			fileMenu.add(getSaveMenuItem());
			fileMenu.add(getExitMenuItem());
		}
		return fileMenu;
	}

	/**
	 * This method initializes jMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getEditMenu() {
		if (editMenu == null) {
			editMenu = new JMenu();
			editMenu.setText("Edit");
			editMenu.add(getCutMenuItem());
			editMenu.add(getCopyMenuItem());
			editMenu.add(getPasteMenuItem());
		}
		return editMenu;
	}

	/**
	 * This method initializes jMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getHelpMenu() {
		if (helpMenu == null) {
			helpMenu = new JMenu();
			helpMenu.setText("Help");
			helpMenu.add(getAboutMenuItem());
		}
		return helpMenu;
	}

	/**
	 * This method initializes jMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getExitMenuItem() {
		if (exitMenuItem == null) {
			exitMenuItem = new JMenuItem();
			exitMenuItem.setText("Exit");
			exitMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					System.exit(0);
				}
			});
		}
		return exitMenuItem;
	}

	/**
	 * This method initializes jMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getAboutMenuItem() {
		if (aboutMenuItem == null) {
			aboutMenuItem = new JMenuItem();
			aboutMenuItem.setText("About");
			aboutMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					JDialog aboutDialog = getAboutDialog();
					aboutDialog.pack();
					Point loc = getJFrame().getLocation();
					loc.translate(20, 20);
					aboutDialog.setLocation(loc);
					aboutDialog.setVisible(true);
				}
			});
		}
		return aboutMenuItem;
	}

	/**
	 * This method initializes aboutDialog	
	 * 	
	 * @return javax.swing.JDialog
	 */
	private JDialog getAboutDialog() {
		if (aboutDialog == null) {
			aboutDialog = new JDialog(getJFrame(), true);
			aboutDialog.setTitle("About");
			aboutDialog.setContentPane(getAboutContentPane());
		}
		return aboutDialog;
	}

	/**
	 * This method initializes aboutContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getAboutContentPane() {
		if (aboutContentPane == null) {
			aboutContentPane = new JPanel();
			aboutContentPane.setLayout(new BorderLayout());
			aboutContentPane.add(getAboutVersionLabel(), BorderLayout.CENTER);
		}
		return aboutContentPane;
	}

	/**
	 * This method initializes aboutVersionLabel	
	 * 	
	 * @return javax.swing.JLabel	
	 */
	private JLabel getAboutVersionLabel() {
		if (aboutVersionLabel == null) {
			aboutVersionLabel = new JLabel();
			aboutVersionLabel.setText("Version 1.0");
			aboutVersionLabel.setHorizontalAlignment(SwingConstants.CENTER);
		}
		return aboutVersionLabel;
	}

	/**
	 * This method initializes jMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getCutMenuItem() {
		if (cutMenuItem == null) {
			cutMenuItem = new JMenuItem();
			cutMenuItem.setText("Cut");
			cutMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
					Event.CTRL_MASK, true));
		}
		return cutMenuItem;
	}

	/**
	 * This method initializes jMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getCopyMenuItem() {
		if (copyMenuItem == null) {
			copyMenuItem = new JMenuItem();
			copyMenuItem.setText("Copy");
			copyMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,
					Event.CTRL_MASK, true));
		}
		return copyMenuItem;
	}

	/**
	 * This method initializes jMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getPasteMenuItem() {
		if (pasteMenuItem == null) {
			pasteMenuItem = new JMenuItem();
			pasteMenuItem.setText("Paste");
			pasteMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,
					Event.CTRL_MASK, true));
		}
		return pasteMenuItem;
	}

	/**
	 * This method initializes jMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getSaveMenuItem() {
		if (saveMenuItem == null) {
			saveMenuItem = new JMenuItem();
			saveMenuItem.setText("Save");
			saveMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
					Event.CTRL_MASK, true));
		}
		return saveMenuItem;
	}

	/**
	 * This method initializes ViewMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getViewMenu() {
		if (ViewMenu == null) {
			ViewMenu = new JMenu();
			ViewMenu.setText("View");
		}
		return ViewMenu;
	}

	/**
	 * This method initializes NewMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getNewMenuItem() {
		if (NewMenuItem == null) {
			NewMenuItem = new JMenuItem();
			NewMenuItem.setText("New");
		}
		return NewMenuItem;
	}

	/**
	 * This method initializes openMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getOpenMenu() {
		if (openMenu == null) {
			openMenu = new JMenu();
			openMenu.setText("Open");
			openMenu.setMnemonic(KeyEvent.VK_O);
			openMenu.add(getOpenBandMenuItem());
			openMenu.add(getOpenMusicianMenuItem());
		}
		return openMenu;
	}

	/**
	 * This method initializes openBandMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getOpenBandMenuItem() {
		if (openBandMenuItem == null) {
			openBandMenuItem = new JMenuItem();
			openBandMenuItem.setText("Band");
		}
		return openBandMenuItem;
	}

	/**
	 * This method initializes openMusicianMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getOpenMusicianMenuItem() {
		if (openMusicianMenuItem == null) {
			openMusicianMenuItem = new JMenuItem();
			openMusicianMenuItem.setText("Musician");
		}
		return openMusicianMenuItem;
	}

	/**
     * This method initializes northPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getNorthPanel() {
        if (northPanel == null) {
            northPanel = new JPanel();
            northPanel.setLayout(new FlowLayout());
            northPanel.add(getStartButton(), null);
        }
        return northPanel;
    }

    /**
     * This method initializes westPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getWestPanel() {
        if (westPanel == null) {
            artistLabel = new JLabel();
            artistLabel.setText("Artist");
            artistLabel.setAlignmentY(0.0F);
            westPanel = new JPanel();
            westPanel.setLayout(new BoxLayout(getWestPanel(), BoxLayout.Y_AXIS));
            westPanel.setPreferredSize(new Dimension(115, 0));
            westPanel.setAlignmentX(0.0F);
            westPanel.setAlignmentY(0.0F);
            westPanel.setMaximumSize(new Dimension(200, 41));
            westPanel.setMinimumSize(new Dimension(115, 41));
            westPanel.setVisible(false);
            westPanel.add(artistLabel, null);
//            westPanel.add(Box.createRigidArea(new Dimension(0,5)));
            westPanel.add(getArtistComboBox(), null);
        }
        return westPanel;
    }

    /**
     * This method initializes imageScrollPane	
     * 	
     * @return javax.swing.JScrollPane	
     */
    private JScrollPane getImageScrollPane() {
        if (imageScrollPane == null) {
            imageLabel = new JLabel();
            imageLabel.setText("");
            imageScrollPane = new JScrollPane();
            imageScrollPane.setViewportView(imageLabel);
        }
        return imageScrollPane;
    }

    /**
     * This method initializes eastPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getEastPanel() {
        if (eastPanel == null) {
            GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
            gridBagConstraints11.fill = GridBagConstraints.BOTH;
            gridBagConstraints11.weighty = 1.0;
            gridBagConstraints11.insets = new Insets(0, 5, 0, 5);
            gridBagConstraints11.weightx = 1.0;
            eastPanel = new JPanel();
            eastPanel.setLayout(new GridBagLayout());
            eastPanel.setPreferredSize(new Dimension(200, 0));
            eastPanel.setVisible(false);
            eastPanel.add(getEastScrollPane(), gridBagConstraints11);
        }
        return eastPanel;
    }

    /**
     * This method initializes southPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getSouthPanel() {
        if (southPanel == null) {
            southPanel = new JPanel();
            southPanel.setLayout(new FlowLayout());
            southPanel.setVisible(false);
            southPanel.add(getPreviousButton(), null);
            southPanel.add(getNextButton(), null);
        }
        return southPanel;
    }

    /**
     * This method initializes startButton	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getStartButton() {
        if (startButton == null) {
            startButton = new JButton();
            startButton.setText("Start");
            startButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    if (startButton.getText().equals("Start")) {
                        startButton.setText("Stop");
                        westPanel.setVisible(true);
                        eastPanel.setVisible(true);
                        southPanel.setVisible(true);
                    } else {
                        System.exit(0);
                    }
                }
            });
        }
        return startButton;
    }

    /**
     * This method initializes previousButton	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getPreviousButton() {
        if (previousButton == null) {
            previousButton = new JButton();
            previousButton.setText("<< Previous");
            previousButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    if (selectedIndex > 0) {
                        artistComboBox.setSelectedIndex(--selectedIndex);
                        disableButtonIfNeeded();
                        setImageAndInfo(selectedIndex);
                    }
                }
            });
        }
        return previousButton;
    }

    /**
     * This method initializes nextButton	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getNextButton() {
        if (nextButton == null) {
            nextButton = new JButton();
            nextButton.setText("Next >>");
            nextButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    if (selectedIndex < (artistComboBox.getItemCount() - 1)) {
                        artistComboBox.setSelectedIndex(++selectedIndex);
                        disableButtonIfNeeded();
                        setImageAndInfo(selectedIndex);
                    }
                }
            });
        }
        return nextButton;
    }

    /**
     * This method initializes artistComboBox	
     * 	
     * @return javax.swing.JComboBox	
     */
    private JComboBox getArtistComboBox() {
        if (artistComboBox == null) {
            artistComboBox = new JComboBox();
            artistComboBox.setAlignmentX(0.0F);
            artistComboBox.setPreferredSize(new Dimension(110, 25));
            artistComboBox.setMinimumSize(new Dimension(110, 25));
            artistComboBox.setMaximumSize(new Dimension(110, 25));
            artistComboBox.setAlignmentY(0.0F);

            Object [] o = {"CSNY", "Norah Jones", "Patti Smith", "Eric Clapton", "Kurt Cobain"};
            for (int i = 0; i < o.length; i++) {
                artistComboBox.addItem(o[i]);
            }
            artistComboBox.setSelectedIndex(-1);
            
            artistComboBox.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(java.awt.event.ItemEvent e) {
                    if (e.getStateChange() == ItemEvent.SELECTED) {
                        selectedIndex = artistComboBox.getSelectedIndex();
                        setImageAndInfo(selectedIndex);
                    }
                }
            });
        }
        return artistComboBox;
    }

    /**
     * This method initializes infoTextArea	
     * 	
     * @return javax.swing.JTextArea	
     */
    private JTextArea getInfoTextArea() {
        if (infoTextArea == null) {
            infoTextArea = new JTextArea();
//            try {
//                infoTextArea.setText((new File(".").getCanonicalPath()));
//            } catch (IOException e) {
//                // TODO Auto-generated catch block
//                e.printStackTrace();
//            }
            infoTextArea.setLineWrap(true);
            infoTextArea.setWrapStyleWord(true);
        }
        return infoTextArea;
    }
    
    private String getArtistInfo(String filename) {
        StringBuffer absoluteFilename = new StringBuffer("");
        StringBuffer info = new StringBuffer("");
        String line = null;
        try {
            absoluteFilename.append(new File(".").getCanonicalPath()).append("\\src\\resources\\").append(filename);
            BufferedReader in = new BufferedReader(new FileReader(new File(absoluteFilename.toString())));
            line = in.readLine();
            while (line != null) {
                info = info.append(line + "\n");
                line = in.readLine();
            }
        } catch (FileNotFoundException e) {
            System.err.println("File not found");
        } catch (IOException e) {
            System.err.println("IO exception caught");
        }
        return info.toString();
    }

    private void setImageAndInfo(int i) {
        switch (i) {
        case 0: 
            imageLabel.setIcon(new ImageIcon(getClass().getResource("/resources/CSNY.jpg"))); 
            infoTextArea.setText(getArtistInfo("csny.txt")); 
            break;
        case 1: 
            imageLabel.setIcon(new ImageIcon(getClass().getResource("/resources/Norah Jones.jpg"))); 
            infoTextArea.setText(getArtistInfo("Norah Jones.txt")); 
            break;
        case 2: 
            imageLabel.setIcon(new ImageIcon(getClass().getResource("/resources/Patti Smith 11.jpg"))); 
            infoTextArea.setText(getArtistInfo("Patti Smith.txt")); 
            break;
        case 3: 
            imageLabel.setIcon(new ImageIcon(getClass().getResource("/resources/Eric Clapton.jpg"))); 
            infoTextArea.setText(getArtistInfo("Eric Clapton.txt")); 
            break;
        case 4: 
            imageLabel.setIcon(new ImageIcon(getClass().getResource("/resources/Kurt Cobain.jpg"))); 
            infoTextArea.setText(getArtistInfo("Kurt Cobain.txt")); 
            break;
        }
    }

    /**
     * This method initializes eastScrollPane	
     * 	
     * @return javax.swing.JScrollPane	
     */
    private JScrollPane getEastScrollPane() {
        if (eastScrollPane == null) {
            eastScrollPane = new JScrollPane();
            eastScrollPane.setViewportView(getInfoTextArea());
        }
        return eastScrollPane;
    }

    private void disableButtonIfNeeded() {
        if (selectedIndex == (artistComboBox.getItemCount() - 1)) {
            nextButton.setEnabled(false);
        } else {
            nextButton.setEnabled(true);
        }
        if (selectedIndex == 0) {
            previousButton.setEnabled(false);
        } else {
            previousButton.setEnabled(true);
        }
    }

    /**
	 * Launches this application
	 */
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				GUI application = new GUI();
				application.getJFrame().setVisible(true);
			}
		});
	}

}
